@@ -3,29 +3,28 @@ import logging
 import ipaddress
 from threading import Timer
 
-from hpimdm import Main
 from hpimdm.utils import TYPE_CHECKING
 from hpimdm.tree.metric import AssertMetric
-from hpimdm.tree import protocol_globals
-from hpimdm.Packet.Packet import Packet
-if protocol_globals.MSG_FORMAT == "BINARY":
-    from hpimdm.Packet.PacketProtocolAck import PacketNewProtocolAck as PacketProtocolAck
-    from hpimdm.Packet.PacketProtocolSync import PacketNewProtocolSync as PacketProtocolHelloSync
-    from hpimdm.Packet.PacketProtocolHelloOptions import PacketNewProtocolHelloHoldtime as PacketProtocolHelloHoldtime
-    from hpimdm.Packet.PacketProtocolHeader import PacketNewProtocolHeader as PacketProtocolHeader
+from hpimdm.tree import hpim_globals
+from hpimdm.tree.hpim_globals import HELLO_PERIOD
+from hpimdm.packet.Packet import Packet
+if hpim_globals.MSG_FORMAT == "BINARY":
+    from hpimdm.packet.PacketHPIMSync import PacketHPIMSync
+    from hpimdm.packet.PacketHPIMHelloOptions import PacketHPIMHelloHoldtime
+    from hpimdm.packet.PacketHPIMHeader import PacketHPIMHeader
 else:
-    from hpimdm.Packet.PacketProtocolAck import PacketProtocolAck
-    from hpimdm.Packet.PacketProtocolSync import PacketProtocolHelloSync
-    from hpimdm.Packet.PacketProtocolHelloOptions import PacketProtocolHelloHoldtime
-    from hpimdm.Packet.PacketProtocolHeader import PacketProtocolHeader
+    from hpimdm.packet.PacketHPIMSync import PacketHPIMSyncJson as PacketHPIMSync
+    from hpimdm.packet.PacketHPIMHelloOptions import PacketHPIMHelloHoldtimeJson as PacketHPIMHelloHoldtime
+    from hpimdm.packet.PacketHPIMHeader import PacketHPIMHeaderJson as PacketHPIMHeader
 
 if TYPE_CHECKING:
-    from hpimdm.InterfaceProtocol import InterfaceProtocol
+    from hpimdm.InterfaceHPIM import InterfaceHPIM
 
-DEFAULT_HELLO_HOLD_TIME_DURING_SYNC = 4 * protocol_globals.SYNC_RETRANSMISSION_TIME
+DEFAULT_HELLO_HOLD_TIME_DURING_SYNC = 4 * hpim_globals.SYNC_RETRANSMISSION_TIME
 DEFAULT_HELLO_HOLD_TIME_AFTER_SYNC = 120
 
-class NeighborState():
+
+class NeighborState:
     @staticmethod
     def new_neighbor_or_adjacency_reset(neighbor):
         neighbor.set_sync_state(Slave)
@@ -45,12 +44,11 @@ class NeighborState():
         my_more_bit = len(neighbor.my_snapshot_multicast_routing_table) > 0
         my_snapshot_sn = neighbor.my_snapshot_sequencer
 
-        pkt_s = PacketProtocolHelloSync(my_snapshot_sn, 0,
-                                        sync_sn=neighbor.current_sync_sn,
-                                        upstream_trees=my_snapshot_mrt, master_flag=True,
-                                        more_flag=my_more_bit,
-                                        neighbor_boot_time=neighbor.time_of_boot)
-        pkt = Packet(payload=PacketProtocolHeader(pkt_s, neighbor.my_snapshot_boot_time))
+        pkt_s = PacketHPIMSync(my_snapshot_sn, 0,
+                               sync_sn=neighbor.current_sync_sn, upstream_trees=my_snapshot_mrt,
+                               master_flag=True, more_flag=my_more_bit,
+                               neighbor_boot_time=neighbor.time_of_boot)
+        pkt = Packet(payload=PacketHPIMHeader(pkt_s, neighbor.my_snapshot_boot_time))
         neighbor.contact_interface.send(pkt, neighbor.ip)
         neighbor.set_sync_timer()
         neighbor.set_hello_hold_time(DEFAULT_HELLO_HOLD_TIME_DURING_SYNC)
@@ -77,11 +75,11 @@ class Synced(NeighborState):
             return
 
         if master_bit and neighbor.my_snapshot_sequencer == my_snapshot_sn:
-            pkt_s = PacketProtocolHelloSync(my_snapshot_sn, neighbor_snapshot_sn, sync_sn=sync_sn,
-                                            master_flag=False, more_flag=False,
-                                            neighbor_boot_time=neighbor.time_of_boot)
-            pkt_s.add_hello_option(PacketProtocolHelloHoldtime(holdtime=4 * neighbor.contact_interface.HELLO_PERIOD))
-            pkt = Packet(payload=PacketProtocolHeader(pkt_s, neighbor.my_snapshot_boot_time))
+            pkt_s = PacketHPIMSync(my_snapshot_sn, neighbor_snapshot_sn, sync_sn=sync_sn,
+                                   master_flag=False, more_flag=False,
+                                   neighbor_boot_time=neighbor.time_of_boot)
+            pkt_s.add_hello_option(PacketHPIMHelloHoldtime(holdtime=4 * HELLO_PERIOD))
+            pkt = Packet(payload=PacketHPIMHeader(pkt_s, neighbor.my_snapshot_boot_time))
             neighbor.send(pkt)
 
 
@@ -109,13 +107,12 @@ class Master(NeighborState):
             my_snapshot_sn = neighbor.my_snapshot_sequencer
             neighbor_sn = neighbor.neighbor_snapshot_sn
 
-            pkt_s = PacketProtocolHelloSync(my_snapshot_sn, neighbor_sn,
-                                            sync_sn=neighbor.current_sync_sn,
-                                            upstream_trees=my_snapshot_mrt, master_flag=False,
-                                            more_flag=my_more_bit, neighbor_boot_time=neighbor.time_of_boot)
+            pkt_s = PacketHPIMSync(my_snapshot_sn, neighbor_sn, sync_sn=neighbor.current_sync_sn,
+                                   upstream_trees=my_snapshot_mrt, master_flag=False, more_flag=my_more_bit,
+                                   neighbor_boot_time=neighbor.time_of_boot)
             if not my_more_bit:
-                pkt_s.add_hello_option(PacketProtocolHelloHoldtime(holdtime=4 * neighbor.contact_interface.HELLO_PERIOD))
-            pkt = Packet(payload=PacketProtocolHeader(pkt_s, neighbor.my_snapshot_boot_time))
+                pkt_s.add_hello_option(PacketHPIMHelloHoldtime(holdtime=4 * HELLO_PERIOD))
+            pkt = Packet(payload=PacketHPIMHeader(pkt_s, neighbor.my_snapshot_boot_time))
             neighbor.send(pkt)
 
             if sync_sn > 0 and not more_bit and not my_more_bit:
@@ -139,14 +136,13 @@ class Master(NeighborState):
         my_snapshot_sn = neighbor.my_snapshot_sequencer
         neighbor_sn = neighbor.neighbor_snapshot_sn
 
-        pkt_s = PacketProtocolHelloSync(my_snapshot_sn, neighbor_sn,
-                                        sync_sn=neighbor.current_sync_sn - 1,
-                                        upstream_trees=my_snapshot_mrt, master_flag=False,
-                                        more_flag=my_more_bit, neighbor_boot_time=neighbor.time_of_boot)
+        pkt_s = PacketHPIMSync(my_snapshot_sn, neighbor_sn, sync_sn=neighbor.current_sync_sn - 1,
+                               upstream_trees=my_snapshot_mrt, master_flag=False, more_flag=my_more_bit,
+                               neighbor_boot_time=neighbor.time_of_boot)
         if not my_more_bit:
-            pkt_s.add_hello_option(PacketProtocolHelloHoldtime(holdtime=4 * neighbor.contact_interface.HELLO_PERIOD))
+            pkt_s.add_hello_option(PacketHPIMHelloHoldtime(holdtime=4 * HELLO_PERIOD))
 
-        pkt = Packet(payload=PacketProtocolHeader(pkt_s, neighbor.my_snapshot_boot_time))
+        pkt = Packet(payload=PacketHPIMHeader(pkt_s, neighbor.my_snapshot_boot_time))
         neighbor.send(pkt)
         neighbor.set_sync_timer()
 
@@ -198,13 +194,12 @@ class Slave(NeighborState):
                 my_snapshot_sn = neighbor.my_snapshot_sequencer
                 neighbor_sn = neighbor.neighbor_snapshot_sn
 
-                pkt_s = PacketProtocolHelloSync(my_snapshot_sn, neighbor_sn,
-                                                sync_sn=neighbor.current_sync_sn,
-                                                upstream_trees=my_snapshot_mrt, master_flag=True,
-                                                more_flag=my_more_bit, neighbor_boot_time=neighbor.time_of_boot)
+                pkt_s = PacketHPIMSync(my_snapshot_sn, neighbor_sn, sync_sn=neighbor.current_sync_sn,
+                                       upstream_trees=my_snapshot_mrt, master_flag=True, more_flag=my_more_bit,
+                                       neighbor_boot_time=neighbor.time_of_boot)
                 if not my_more_bit:
-                    pkt_s.add_hello_option(PacketProtocolHelloHoldtime(holdtime=4 * neighbor.contact_interface.HELLO_PERIOD))
-                pkt = Packet(payload=PacketProtocolHeader(pkt_s, neighbor.my_snapshot_boot_time))
+                    pkt_s.add_hello_option(PacketHPIMHelloHoldtime(holdtime=4 * HELLO_PERIOD))
+                pkt = Packet(payload=PacketHPIMHeader(pkt_s, neighbor.my_snapshot_boot_time))
                 neighbor.send(pkt)
                 neighbor.set_sync_timer()
 
@@ -216,14 +211,13 @@ class Slave(NeighborState):
         my_snapshot_sn = neighbor.my_snapshot_sequencer
         neighbor_sn = neighbor.neighbor_snapshot_sn
 
-        pkt_s = PacketProtocolHelloSync(my_snapshot_sn, neighbor_sn,
-                                        sync_sn=neighbor.current_sync_sn,
-                                        upstream_trees=my_snapshot_mrt, master_flag=True,
-                                        more_flag=my_more_bit, neighbor_boot_time=neighbor.time_of_boot)
+        pkt_s = PacketHPIMSync(my_snapshot_sn, neighbor_sn, sync_sn=neighbor.current_sync_sn,
+                               upstream_trees=my_snapshot_mrt, master_flag=True, more_flag=my_more_bit,
+                               neighbor_boot_time=neighbor.time_of_boot)
         if not my_more_bit:
-            pkt_s.add_hello_option(PacketProtocolHelloHoldtime(holdtime=4 * neighbor.contact_interface.HELLO_PERIOD))
+            pkt_s.add_hello_option(PacketHPIMHelloHoldtime(holdtime=4 * HELLO_PERIOD))
 
-        pkt = Packet(payload=PacketProtocolHeader(pkt_s, neighbor.my_snapshot_boot_time))
+        pkt = Packet(payload=PacketHPIMHeader(pkt_s, neighbor.my_snapshot_boot_time))
         neighbor.send(pkt)
         neighbor.set_sync_timer()
 
@@ -252,11 +246,11 @@ class Unknown(NeighborState):
 
 
 class Neighbor:
-    LOGGER = logging.getLogger('protocol.Interface.Neighbor')
+    LOGGER = logging.getLogger('hpim.Interface.Neighbor')
 
-    def __init__(self, contact_interface: "InterfaceProtocol", ip, hello_hold_time: int, neighbor_time_of_boot: int,
+    def __init__(self, contact_interface: "InterfaceHPIM", ip, hello_hold_time: int, neighbor_time_of_boot: int,
                  my_interface_boot_time: int):
-        if hello_hold_time == protocol_globals.HELLO_HOLD_TIME_TIMEOUT:
+        if hello_hold_time == hpim_globals.HELLO_HOLD_TIME_TIMEOUT:
             raise Exception
         logger_info = dict(contact_interface.interface_logger.extra)
         logger_info['neighbor_ip'] = ip
@@ -303,7 +297,7 @@ class Neighbor:
         Set Sync timer... useful when the Sync process is making progress and a Sync message from the neighbor node must be received
         """
         self.clear_sync_timer()
-        self.sync_timer = Timer(protocol_globals.SYNC_RETRANSMISSION_TIME, self.sync_timeout)
+        self.sync_timer = Timer(hpim_globals.SYNC_RETRANSMISSION_TIME, self.sync_timeout)
         self.sync_timer.start()
 
     def clear_sync_timer(self):
@@ -333,7 +327,7 @@ class Neighbor:
         if self.neighbor_liveness_timer is not None:
             self.neighbor_liveness_timer.cancel()
 
-        if hello_hold_time == protocol_globals.HELLO_HOLD_TIME_TIMEOUT:
+        if hello_hold_time == hpim_globals.HELLO_HOLD_TIME_TIMEOUT:
             self.remove()
         else:
             self.neighbor_liveness_timer = Timer(hello_hold_time, self.remove)
@@ -368,7 +362,7 @@ class Neighbor:
                                    '; NeighborBootTime=' + str(self.time_of_boot) +
                                    '; NeighborSnapshotSN=' + str(self.neighbor_snapshot_sn))
         if state == Synced:
-            Main.kernel.recheck_all_trees(self.contact_interface.vif_index)
+            self.contact_interface.get_kernel().recheck_all_trees(self.contact_interface.vif_index)
 
     def install_tree_state(self, tree_state: list):
         """
@@ -469,11 +463,11 @@ class Neighbor:
             return False
         elif sn >= last_received_sn:
             (source, group) = tree
-            ack = PacketProtocolAck(source, group, sn, neighbor_boot_time=boot_time,
-                                    neighbor_snapshot_sn=self.neighbor_snapshot_sn,
-                                    my_snapshot_sn=self.my_snapshot_sequencer)
-            ph = PacketProtocolHeader(ack, boot_time=self.contact_interface.time_of_boot)
-            packet = Packet(payload=ph)
+            packet = self.contact_interface.create_ack_msg(my_boot_time=self.contact_interface.time_of_boot, sn=sn,
+                                                           source=source, group=group,
+                                                           neighbor_boot_time=boot_time,
+                                                           neighbor_snapshot_sn=self.neighbor_snapshot_sn,
+                                                           my_snapshot_sn=self.my_snapshot_sequencer)
             self.contact_interface.send(packet, self.ip)
 
             if sn > last_received_sn:
@@ -536,12 +530,13 @@ class Neighbor:
         self.my_snapshot_boot_time = my_snapshot_bt
         self.my_snapshot_sequencer = my_snapshot_sn
         self.my_snapshot_multicast_routing_table = list(my_snapshot_mrt.values())
-        self.sync_fragmentation = protocol_globals.SYNC_FRAGMENTATION_MSG
+        self.sync_fragmentation = hpim_globals.SYNC_FRAGMENTATION_MSG
         if self.sync_fragmentation == 0:
-            self.sync_fragmentation = (self.contact_interface.get_mtu() - 20 - 8 - 16) // 16
+            sync_entry_len = len(self.contact_interface.create_sync_entry_hdr("", "", 0, 0))
+            self.sync_fragmentation = (self.contact_interface.get_mtu() - self.contact_interface.get_ip_header_length()
+                                       - 8 - self.contact_interface.security_len - 16) // sync_entry_len
         self.contact_interface.neighbor_start_synchronization(self.ip, my_snapshot_bt, my_snapshot_sn)
 
-
     #################################################################
     # Obtain Upstream and Interest information regarding a neighbor
     #################################################################
@@ -557,12 +552,11 @@ class Neighbor:
             upstream_state = self.tree_metric_state.get(tree, None)
             interest_state = False
             if upstream_state is None:
-                interest_state = self.tree_interest_state.get(tree, protocol_globals.INITIAL_FLOOD_ENABLED)
+                interest_state = self.tree_interest_state.get(tree, hpim_globals.INITIAL_FLOOD_ENABLED)
             print("INTEREST NEIGHBOR ", self.ip, ":", interest_state)
             print("UPSTREAM NEIGHBOR ", self.ip, ":", upstream_state)
             return (interest_state, upstream_state)
 
-
     #######################################
     # Remove state regarding neighbor
     #######################################
